html, body, iframe, div { margin:0; padding:0; }

#epiceditor-utilbar {
  position:fixed;
  bottom:10px;
  right:10px;
  padding:5px;
}

#epiceditor-utilbar img {
  display:block;
  float:left;
  width: 30px;
  height: 30px;
}

#epiceditor-utilbar img:last-child {
  margin-left: 15px;
}

#epiceditor-utilbar img:hover {
  cursor:pointer;
}

.epiceditor-edit-mode #epiceditor-utilbar img.epiceditor-toggle-edit-btn {
  display: none;
}

.epiceditor-preview-mode #epiceditor-utilbar img.epiceditor-toggle-preview-btn {
  display: none;
}
